"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var EventEmitter = (function () {
    function EventEmitter() {
        this._eventListeners = {};
        this._firedEvents = [];
    }
    EventEmitter.prototype.emit = function (type, payload) {
        var event = this.createEvent(type, payload);
        this._firedEvents[type] = event;
        if (!(event.type in this._eventListeners)) {
            return false;
        }
        var subscribers = this._eventListeners[event.type].slice();
        for (var i = 0, l = subscribers.length; i < l; i++) {
            var subscriber = subscribers[i];
            subscriber.fired = true;
            subscriber.callback.call(this, event);
        }
        return true;
    };
    EventEmitter.prototype.on = function (type, callback, config) {
        if (config === void 0) { config = { immediate: false }; }
        if (!(type in this._eventListeners)) {
            this._eventListeners[type] = [];
        }
        this._eventListeners[type].push({
            callback: callback,
            immediate: config.immediate,
        });
        var event = this.createEvent(type);
        if (this._firedEvents[type]) {
            callback(this._firedEvents[type]);
            return;
        }
        this._eventListeners[type]
            .slice()
            .filter(function (subscriber) { return subscriber.fired && subscriber.immediate; })
            .forEach(function (subscriber) { return subscriber.callback(event); });
    };
    EventEmitter.prototype.createEvent = function (type, payload) {
        return {
            type: type,
            target: this,
            payload: payload,
        };
    };
    return EventEmitter;
}());
exports.default = EventEmitter;
//# sourceMappingURL=EventEmitter.js.map