"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("./utils");
var Registry = (function () {
    function Registry() {
        this.entries = new Map();
        if (Registry.instance) {
            return Registry.instance;
        }
        Registry.instance = this;
        return this;
    }
    Registry.getInstance = function () {
        if (!Registry.instance) {
            Registry.instance = new Registry();
        }
        return Registry.instance;
    };
    Object.defineProperty(Registry.prototype, "size", {
        get: function () {
            return Registry.instance.entries.size;
        },
        enumerable: false,
        configurable: true
    });
    Registry.prototype.set = function (key, value) {
        var _a;
        var entries = Array.isArray(key) ? key : [(_a = {}, _a[key] = value, _a)];
        entries.forEach(function (entry) {
            Object.entries(entry).forEach(function (_a) {
                var key = _a[0], value = _a[1];
                return Registry.instance.entries.set(key, value);
            });
        });
    };
    Registry.prototype.get = function (key) {
        if (!Registry.instance.entries.has(key)) {
            if (utils_1.config().debug) {
                console.warn("Registry Error: " + key + " is not available in the registry.");
            }
            return null;
        }
        return Registry.instance.entries.get(key);
    };
    Registry.prototype.has = function (key) {
        return Registry.instance.entries.has(key);
    };
    Registry.prototype.clear = function () {
        Registry.instance.entries.clear();
    };
    Registry.prototype.delete = function (key) {
        return Registry.instance.entries.delete(key);
    };
    Registry.prototype.__jest__destroyInstance = function () {
        if (process.env.NODE_ENV !== 'test')
            return;
        delete Registry.instance;
    };
    return Registry;
}());
exports.default = Registry;
//# sourceMappingURL=Registry.js.map