"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var apollo_link_1 = require("apollo-link");
var PusherLink = (function (_super) {
    tslib_1.__extends(PusherLink, _super);
    function PusherLink(options) {
        var _this = _super.call(this) || this;
        _this.pusher = options.pusher;
        return _this;
    }
    PusherLink.prototype.request = function (operation, forward) {
        var _this = this;
        return new apollo_link_1.Observable(function (observer) {
            forward(operation).subscribe({
                next: function (data) {
                    var subscriptionChannel = _this._getChannel({ data: data, operation: operation });
                    if (subscriptionChannel) {
                        _this._createSubscription(subscriptionChannel, observer);
                    }
                    else {
                        observer.next(data);
                        observer.complete();
                    }
                },
            });
        });
    };
    PusherLink.prototype._getChannel = function (_a) {
        var data = _a.data, operation = _a.operation;
        return !!data.extensions
            && !!data.extensions.lighthouse_subscriptions
            && !!data.extensions.lighthouse_subscriptions.channels
            ? data.extensions.lighthouse_subscriptions.channels[operation.operationName]
            : null;
    };
    PusherLink.prototype._createSubscription = function (subscriptionChannel, observer) {
        var _this = this;
        var pusherChannel = this.pusher.subscribe(subscriptionChannel);
        pusherChannel.bind('lighthouse-subscription', function (payload) {
            if (!payload.more) {
                _this.pusher.unsubscribe(subscriptionChannel);
                observer.complete();
            }
            var result = payload.result;
            if (result) {
                observer.next(result);
            }
        });
    };
    return PusherLink;
}(apollo_link_1.ApolloLink));
exports.default = PusherLink;
//# sourceMappingURL=PusherLink.js.map