"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var lodash_clonedeep_1 = tslib_1.__importDefault(require("lodash.clonedeep"));
var to_case_1 = tslib_1.__importDefault(require("to-case"));
var utils_1 = require("../utils");
var Collection_1 = tslib_1.__importDefault(require("./Collection"));
var EventEmitter_1 = tslib_1.__importDefault(require("../EventEmitter"));
var uuid_1 = require("uuid");
var Model = (function (_super) {
    tslib_1.__extends(Model, _super);
    function Model(props) {
        if (props === void 0) { props = {}; }
        var _this = _super.call(this) || this;
        _this.loading = false;
        _this.submittableProps = [];
        _this.performSafeRequestGraphql = utils_1.performSafeRequestGraphql;
        _this.performSafeRequestREST = utils_1.performSafeRequestREST;
        _this._exists = false;
        Object.assign(_this, _this.defaults(), props);
        Object.assign(_this, props);
        if (typeof _this.hydrateProps === 'function') {
            console.warn('Deprecated: Use property variables or constructor instead. `hydrateProps` will be removed in the next versions.');
            _this.hydrateProps(props);
        }
        return _this;
    }
    Model.prototype.generateUuid = function () {
        this.uuid = uuid_1.v4();
    };
    Object.defineProperty(Model.prototype, "doRequest", {
        get: function () {
            return utils_1.config().graphql ? utils_1.performSafeRequestGraphql : utils_1.performSafeRequestREST;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Model.prototype, "className", {
        get: function () {
            return this.constructor.name;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Model.prototype, "displayClassName", {
        get: function () {
            return this.className.toLowerCase();
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Model.prototype, "defaultMethod", {
        get: function () {
            return 'post';
        },
        enumerable: false,
        configurable: true
    });
    Model.prototype.exists = function () {
        return this._exists || Boolean(this.__typename) || Boolean(this.id);
    };
    Model.prototype.defaults = function () {
        return {};
    };
    Model.prototype.hydrate = function (item) {
        return new this.constructor(item);
    };
    Model.prototype.getMethod = function (_opts) {
        return null;
    };
    Model.prototype.toCollection = function (skipEmpty) {
        if (skipEmpty === void 0) { skipEmpty = false; }
        return (new Collection_1.default(this)).filter(function (i) { return !!i || !skipEmpty; });
    };
    Model.prototype.toArray = function (skipEmpty) {
        if (skipEmpty === void 0) { skipEmpty = false; }
        return this.toCollection(skipEmpty).all();
    };
    Model.prototype.getClassName = function () {
        return this.constructor.name;
    };
    Model.prototype.getInputFields = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2, utils_1.getSchemaTypeFields(this.getClassName() + "Input")];
            });
        });
    };
    Model.prototype.getUpdateVariables = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var keys, result, _i, keys_1, key, _a, _b, _c, _d, _e, _f;
            var _this = this;
            return tslib_1.__generator(this, function (_g) {
                switch (_g.label) {
                    case 0: return [4, this.getInputFields()];
                    case 1:
                        keys = _g.sent();
                        result = {};
                        _i = 0, keys_1 = keys;
                        _g.label = 2;
                    case 2:
                        if (!(_i < keys_1.length)) return [3, 9];
                        key = keys_1[_i];
                        if (!Array.isArray(this[key])) return [3, 4];
                        _a = result;
                        _b = key;
                        return [4, Promise.all(this[key].map(function (model) { return tslib_1.__awaiter(_this, void 0, void 0, function () { var _a; return tslib_1.__generator(this, function (_b) {
                                switch (_b.label) {
                                    case 0:
                                        if (!(model instanceof Model)) return [3, 2];
                                        return [4, model.getUpdateVariables()];
                                    case 1:
                                        _a = _b.sent();
                                        return [3, 4];
                                    case 2: return [4, model];
                                    case 3:
                                        _a = _b.sent();
                                        _b.label = 4;
                                    case 4: return [2, (_a)];
                                }
                            }); }); }))];
                    case 3:
                        _a[_b] = _g.sent();
                        return [3, 8];
                    case 4:
                        if (!(this[key] instanceof Model)) return [3, 6];
                        _c = result;
                        _d = key;
                        return [4, this[key].getUpdateVariables()];
                    case 5:
                        _c[_d] = _g.sent();
                        return [3, 8];
                    case 6:
                        _e = result;
                        _f = key;
                        return [4, this[key]];
                    case 7:
                        _e[_f] = _g.sent();
                        _g.label = 8;
                    case 8:
                        _i++;
                        return [3, 2];
                    case 9: return [2, result];
                }
            });
        });
    };
    Model.prototype.toSubmittable = function (props) {
        if (props === void 0) { props = this.submittableProps; }
        if (!props)
            return [];
        if (Array.isArray(props) && !props.length)
            return [];
        return this.toCollection().only(props).all();
    };
    Model.prototype.toSubmittableFormData = function (props) {
        if (props === void 0) { props = this.submittableProps; }
        var fd = new FormData();
        if (!props || (Array.isArray(props) && !props.length))
            return fd;
        this.toCollection().only(props).each(function (item, index) {
            fd.append(index, item);
        });
        return fd;
    };
    Model.prototype.save = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2, this.exists() ? this.update() : this.create()];
            });
        });
    };
    Model.prototype.create = function (params, mutation) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var resolvedRequest, resolvedParams, _a, _b;
            return tslib_1.__generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        resolvedRequest = this.resolveRequest(mutation, this.createMutation);
                        return [4, this.resolveParams(params)];
                    case 1:
                        resolvedParams = _c.sent();
                        if (!utils_1.config().graphql) return [3, 3];
                        _a = this.hydrate;
                        return [4, this.mutate(resolvedRequest, resolvedParams)];
                    case 2: return [2, _a.apply(this, [_c.sent()])];
                    case 3:
                        _b = this.hydrate;
                        return [4, this.mutate(resolvedRequest, resolvedParams)];
                    case 4: return [2, _b.apply(this, [_c.sent()])];
                }
            });
        });
    };
    Model.prototype.update = function (params, mutation) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var resolvedRequest, resolvedParams, _a, _b;
            return tslib_1.__generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        resolvedRequest = this.resolveRequest(mutation, this.updateMutation);
                        return [4, this.resolveParams(params)];
                    case 1:
                        resolvedParams = _c.sent();
                        if (!utils_1.config().graphql) return [3, 3];
                        _a = this.hydrate;
                        return [4, this.mutate(resolvedRequest, resolvedParams)];
                    case 2: return [2, _a.apply(this, [_c.sent()])];
                    case 3:
                        _b = this.hydrate;
                        return [4, this.mutate(resolvedRequest, resolvedParams)];
                    case 4: return [2, _b.apply(this, [_c.sent()])];
                }
            });
        });
    };
    Model.prototype.delete = function (params, mutation) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var resolvedRequest, forget;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                resolvedRequest = this.resolveRequest(mutation, this.deleteMutation);
                forget = function (result) { return _this.markNotExists() && result; };
                if (utils_1.config().graphql) {
                    return [2, this.mutate(resolvedRequest, params || {
                            uuid: this.uuid,
                        }).then(forget)];
                }
                return [2, this.mutate(resolvedRequest, params || {
                        id: this.id,
                    }, 'delete').then(forget)];
            });
        });
    };
    Model.prototype.mutate = function (mutationOrUrl, params, method) {
        if (method === void 0) { method = this.defaultMethod; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var resolvedUrl, resolvedMethod;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (utils_1.config().graphql) {
                            return [2, this.beforeMutate()
                                    .then(this.performSafeRequestGraphql.bind(this, mutationOrUrl, params, null))
                                    .finally(this.afterMutate.bind(this))
                                    .finally(this.markExists.bind(this))];
                        }
                        return [4, utils_1.getUrl({ method: method, url: mutationOrUrl, params: params })];
                    case 1:
                        resolvedUrl = _a.sent();
                        return [4, this.getMethod({ method: method, url: resolvedUrl, params: params })];
                    case 2:
                        resolvedMethod = _a.sent();
                        return [2, this.beforeMutate()
                                .then(this.performSafeRequestREST.bind(this, resolvedUrl, params, resolvedMethod || method, null))
                                .finally(this.afterMutate.bind(this))
                                .finally(this.markExists.bind(this))];
                }
            });
        });
    };
    Model.prototype.clone = function () {
        return lodash_clonedeep_1.default(this).markNotExists();
    };
    Model.prototype.markExists = function () {
        this._exists = true;
        return this;
    };
    Model.prototype.markNotExists = function () {
        this._exists = false;
        return this;
    };
    Model.prototype.beforeMutate = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                this.loading = true;
                return [2, this];
            });
        });
    };
    Model.prototype.afterMutate = function () {
        this.loading = false;
        return this;
    };
    Model.prototype.resolveRequest = function (request, defaultRequest) {
        if (request)
            return request;
        return typeof defaultRequest === 'function' ? defaultRequest() : defaultRequest;
    };
    Model.prototype.resolveParams = function (params) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var model;
            var _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (!utils_1.config().graphql) return [3, 2];
                        return [4, this.getUpdateVariables()];
                    case 1:
                        model = _b.sent();
                        return [2, (_a = {},
                                _a[to_case_1.default.camel(this.className)] = model,
                                _a)];
                    case 2: return [2, params];
                }
            });
        });
    };
    return Model;
}(EventEmitter_1.default));
exports.default = Model;
//# sourceMappingURL=Model.js.map