"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var graphql_1 = require("graphql");
var InvalidArgumentException_1 = tslib_1.__importDefault(require("../models/Exceptions/InvalidArgumentException"));
var Collection_1 = tslib_1.__importDefault(require("../models/Collection"));
var utils_1 = require("../utils");
var UnexpectedException_1 = tslib_1.__importDefault(require("../models/Exceptions/UnexpectedException"));
var ValidationException_1 = tslib_1.__importDefault(require("../models/Exceptions/ValidationException"));
var UnauthorizedException_1 = tslib_1.__importDefault(require("../models/Exceptions/UnauthorizedException"));
var Model_1 = tslib_1.__importDefault(require("../models/Model"));
var EventEmitter_1 = tslib_1.__importDefault(require("../EventEmitter"));
var Repository = (function (_super) {
    tslib_1.__extends(Repository, _super);
    function Repository(array) {
        if (array === void 0) { array = null; }
        var _this = _super.call(this) || this;
        _this._exists = false;
        _this.performSafeRequestGraphql = utils_1.performSafeRequestGraphql;
        _this.performSafeRequestREST = utils_1.performSafeRequestREST;
        _this.loading = true;
        _this.lastError = null;
        _this.method = '';
        _this.dataset = new Collection_1.default();
        _this.queryParams = {};
        _this.model = function undefinedMethod(data) {
            return Object(data);
        };
        if (array instanceof Collection_1.default) {
            _this.dataset = array;
        }
        else if (Array.isArray(array)) {
            _this.dataset = _this.fromArray(array);
        }
        _this.emit('construct');
        return _this;
    }
    Repository.prototype.subscriptions = function () {
        return [];
    };
    Object.defineProperty(Repository.prototype, "className", {
        get: function () {
            return this.constructor.name;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Repository.prototype, "displayClassName", {
        get: function () {
            return this.className.toLowerCase();
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Repository.prototype, "defaultMethod", {
        get: function () {
            return utils_1.config().graphql ? 'post' : 'get';
        },
        enumerable: false,
        configurable: true
    });
    Repository.prototype.exists = function () {
        return this._exists;
    };
    Repository.prototype.getMethod = function (opts) {
        return null;
    };
    Repository.prototype.fromArray = function (array, skipEmpty) {
        var _this = this;
        if (skipEmpty === void 0) { skipEmpty = true; }
        var filtered = array.filter(function (i) { return i || !skipEmpty; });
        var mapped = filtered.map(function (i) {
            var model = _this.model;
            if (typeof _this.model === 'function' && !utils_1.isClass(_this.model)) {
                model = _this.model(i);
            }
            return new model(i);
        });
        return new Collection_1.default(mapped);
    };
    Repository.fromArray = function (array, skipEmpty) {
        if (skipEmpty === void 0) { skipEmpty = true; }
        return (new this).fromArray(array, skipEmpty);
    };
    Repository.prototype.onError = function (e) {
        this.lastError = e;
        this.emit('onError');
        if (utils_1.config().graphql && (e instanceof graphql_1.GraphQLError)) {
            var _a = e.graphQLErrors, _b = (_a === void 0 ? [
                {
                    extensions: {
                        errorCode: 500,
                        message: 'Server Error',
                    },
                },
            ] : _a)[0].extensions, errorCode = _b.errorCode, message = _b.message;
            if (errorCode === 401) {
                throw new UnauthorizedException_1.default(message);
            }
            if (errorCode === 422) {
                throw new ValidationException_1.default(message);
            }
            throw new UnexpectedException_1.default(e.message);
        }
    };
    Repository.prototype.query = function (queryOrUrl, params, collection, method) {
        if (params === void 0) { params = {}; }
        if (collection === void 0) { collection = false; }
        if (method === void 0) { method = this.defaultMethod; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var doc, resolvedUrl, resolvedMethod;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!utils_1.config().graphql) return [3, 3];
                        doc = queryOrUrl;
                        if (!(typeof queryOrUrl === 'function')) return [3, 2];
                        return [4, queryOrUrl()];
                    case 1:
                        doc = (_a.sent());
                        _a.label = 2;
                    case 2: return [2, this.beforeQuery()
                            .then(this.performSafeRequestGraphql.bind(this, doc, params, this.subscriptions()))
                            .then(this.processResponse.bind(this, collection))
                            .catch(this.onError.bind(this))
                            .finally(this.afterQuery.bind(this))];
                    case 3:
                        this.method = method;
                        return [4, utils_1.getUrl({ method: method, url: queryOrUrl, params: params })];
                    case 4:
                        resolvedUrl = _a.sent();
                        return [4, this.getMethod({ method: method, url: resolvedUrl, params: params })];
                    case 5:
                        resolvedMethod = _a.sent();
                        return [2, this.beforeQuery()
                                .then(this.performSafeRequestREST.bind(this, resolvedUrl, params || this.queryParams, resolvedMethod || method, null))
                                .then(this.processResponse.bind(this, collection))
                                .catch(this.onError.bind(this))
                                .finally(this.afterQuery.bind(this))];
                }
            });
        });
    };
    Repository.prototype.beforeQuery = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                this.loading = true;
                this.emit('beforeQuery');
                return [2];
            });
        });
    };
    Repository.prototype.processResponse = function (collection, data) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2, collection ? this.fromArray(data) : data];
            });
        });
    };
    Repository.prototype.afterQuery = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                this.loading = false;
                this.emit('afterQuery');
                return [2];
            });
        });
    };
    Repository.prototype.many = function (params) {
        if (params === void 0) { params = null; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _a;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (!this.fetchManyQuery) {
                            throw new InvalidArgumentException_1.default('fetchAllQuery is not set for this repository.');
                        }
                        _a = this;
                        return [4, this.query(this.fetchManyQuery, params || this.queryParams, true)];
                    case 1:
                        _a.dataset = _b.sent();
                        this._exists = true;
                        return [2, this.dataset];
                }
            });
        });
    };
    Repository.prototype.one = function (id) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var params, data, ModelFactory;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        params = utils_1.config().graphql ? { uuid: id } : { id: id };
                        return [4, this.query(this.fetchOneQuery, params)];
                    case 1:
                        data = _a.sent();
                        ModelFactory = this.model;
                        if (typeof ModelFactory === 'function' && !(ModelFactory.prototype instanceof Model_1.default)) {
                            ModelFactory = ModelFactory(data);
                        }
                        return [2, new ModelFactory(data)];
                }
            });
        });
    };
    Repository.one = function (params) {
        return Reflect.construct(this, []).one(params);
    };
    return Repository;
}(EventEmitter_1.default));
exports.default = Repository;
//# sourceMappingURL=Repository.js.map