"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isClass = exports.fetchIntrospectionSchema = exports.stripObject = exports.getUrl = exports.getSchemaQuery = exports.getSchemaMutation = exports.getSchemaTypeFields = exports.getParsedSchema = exports.config = exports.registryGet = exports.performSafeRequestGraphql = exports.stripTypename = exports.performGqlMutation = exports.performGqlQuery = exports.performSafeRequestREST = exports.json = exports.status = exports.queryParams = exports.camelToKebab = exports.getApolloManager = exports.defaultRESTHeaders = void 0;
var tslib_1 = require("tslib");
var graphql_1 = require("graphql");
var parser_1 = require("graphql/language/parser");
var graphql_2 = require("graphql");
var Registry_1 = tslib_1.__importDefault(require("./Registry"));
var UnexpectedException_1 = tslib_1.__importDefault(require("./models/Exceptions/UnexpectedException"));
var apolloClient_1 = require("./graphql/apolloClient");
exports.defaultRESTHeaders = {
    'Accept': 'application/json',
    'Content-Type': 'application/json;charset=UTF-8',
};
function getApolloManager() {
    return apolloClient_1.apolloClient.queryManager;
}
exports.getApolloManager = getApolloManager;
function camelToKebab(input) {
    return input.replace(/([a-z])([A-Z])/g, '$1-$2').toLowerCase();
}
exports.camelToKebab = camelToKebab;
function queryParams(params, camelToKebabActive) {
    if (camelToKebabActive === void 0) { camelToKebabActive = true; }
    var str = [];
    for (var paramsKey in params)
        if (params.hasOwnProperty(paramsKey) && (params[paramsKey] || params[paramsKey] === 0)) {
            var normalized = camelToKebabActive ? camelToKebab(encodeURIComponent(paramsKey)) : encodeURIComponent(paramsKey);
            str.push(normalized + "=" + encodeURIComponent(params[paramsKey]));
        }
    return str.join('&');
}
exports.queryParams = queryParams;
function status(res) {
    var response = res.hasOwnProperty('status') ? res : res.response;
    if (response.status && response.status >= 200 && response.status < 300) {
        return Promise.resolve(response);
    }
    else {
        if (response.data && response.data.errors) {
            var errors = (response.data.errors || []).map(function (err) { return err.defaultMessage; }).join(', ');
            return Promise.reject(new Error(errors));
        }
        return Promise.reject(new Error(response.data.message));
    }
}
exports.status = status;
function json(response) {
    if (response && response.data) {
        return response.data;
    }
    else {
        return null;
    }
}
exports.json = json;
function performSafeRequestREST(url, params, method, opts) {
    if (params === void 0) { params = {}; }
    if (method === void 0) { method = 'get'; }
    if (opts === void 0) { opts = {}; }
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var fullUrl, body, queryEscaped;
        return tslib_1.__generator(this, function (_a) {
            fullUrl = url;
            body = {};
            if (method === 'get') {
                queryEscaped = queryParams(params);
                if (queryEscaped) {
                    fullUrl = fullUrl + "?" + queryEscaped;
                }
            }
            else {
                body = params;
            }
            return [2, fetch(fullUrl, tslib_1.__assign({ method: method.toLowerCase(), headers: exports.defaultRESTHeaders, body: JSON.stringify(body) }, opts)).then(status).then(json)];
        });
    });
}
exports.performSafeRequestREST = performSafeRequestREST;
function performGqlQuery(query, variables) {
    return getApolloManager().query({
        query: query,
        variables: variables,
    });
}
exports.performGqlQuery = performGqlQuery;
function performGqlMutation(mutation, variables) {
    return getApolloManager().mutate({
        mutation: mutation,
        variables: variables,
    });
}
exports.performGqlMutation = performGqlMutation;
function stripTypename(obj) {
    return JSON.parse(JSON.stringify(obj, function (k, v) { return (k === '__typename' ? undefined : v); }));
}
exports.stripTypename = stripTypename;
function performSafeRequestGraphql(query, variables) {
    if (variables === void 0) { variables = {}; }
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var queryName, isQuery;
        return tslib_1.__generator(this, function (_a) {
            queryName = query.definitions.map(function (def) { return def.name; }).find(function (def) { return def.kind === 'Name'; }).value;
            isQuery = query.definitions.find(function (def) { return def.kind === 'OperationDefinition'; }).operation === 'query';
            if (isQuery) {
                return [2, performGqlQuery(query, stripTypename(variables))
                        .then(function (value) { return value.data[queryName]; })];
            }
            return [2, performGqlMutation(query, stripTypename(variables))
                    .then(function (value) { return value.data[queryName]; })];
        });
    });
}
exports.performSafeRequestGraphql = performSafeRequestGraphql;
function registryGet(key) {
    return Registry_1.default.getInstance().get(key);
}
exports.registryGet = registryGet;
function config() {
    return registryGet('Config');
}
exports.config = config;
function getParsedSchema() {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var configSchema, configSchemaUrl, schema;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    configSchema = config().schema;
                    configSchemaUrl = config().schemaUrl;
                    schema = Registry_1.default.getInstance().get('schema');
                    if (!(!schema && configSchema)) return [3, 1];
                    schema = configSchema;
                    return [3, 3];
                case 1:
                    if (!(!schema && configSchemaUrl)) return [3, 3];
                    return [4, fetchIntrospectionSchema(configSchemaUrl)
                            .then(graphql_1.buildClientSchema.bind(null))
                            .then(graphql_1.printSchema.bind(null))
                            .then(parser_1.parse.bind(null))];
                case 2:
                    schema = _a.sent();
                    Registry_1.default.getInstance().set('schema', schema);
                    _a.label = 3;
                case 3:
                    if (!schema) {
                        throw new UnexpectedException_1.default('Configuration error: \'schema\' must be passed as a config key, e.g\n\nimport schema from \'raw-loader!@/../schema.graphql\';\n\n//...\n\nVue.use(VueOOP, {\n  //...,\n  schema,\n})\n\n;');
                    }
                    return [2, schema];
            }
        });
    });
}
exports.getParsedSchema = getParsedSchema;
function getSchemaTypeFields(typeName) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, getParsedSchema()];
                case 1: return [2, (_a.sent())
                        .definitions
                        .find(function (def) { return (def.name || {}).value === typeName; })
                        .fields
                        .map(function (f) { return f.name.value; })];
            }
        });
    });
}
exports.getSchemaTypeFields = getSchemaTypeFields;
function getSchemaMutation(mutationName) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, getParsedSchema()];
                case 1: return [2, (_a.sent())
                        .definitions
                        .find(function (def) { return (def.name || {}).value === 'Mutation'; })
                        .fields
                        .find(function (def) { return (def.name || {}).value === mutationName; })];
            }
        });
    });
}
exports.getSchemaMutation = getSchemaMutation;
function getSchemaQuery(queryName) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, getParsedSchema()];
                case 1: return [2, (_a.sent())
                        .definitions
                        .find(function (def) { return (def.name || {}).value === 'Query'; })
                        .fields
                        .find(function (def) { return (def.name || {}).value === queryName; })];
            }
        });
    });
}
exports.getSchemaQuery = getSchemaQuery;
function getUrl(_opts) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var url, params, resolvedUrl;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    url = _opts.url, params = _opts.params;
                    resolvedUrl = url;
                    if (!(typeof url === 'function')) return [3, 2];
                    return [4, url()];
                case 1:
                    resolvedUrl = _a.sent();
                    return [3, 3];
                case 2:
                    resolvedUrl = resolvedUrl.replace(/:([^\s\/?&]+)/gi, function (_, m) {
                        var param = params[m];
                        var hasParam = param !== undefined;
                        if (hasParam) {
                            delete params[m];
                            return param;
                        }
                        return m;
                    });
                    _a.label = 3;
                case 3: return [2, resolvedUrl];
            }
        });
    });
}
exports.getUrl = getUrl;
function stripObject(obj) {
    return JSON.parse(JSON.stringify(obj, function (k, v) { return (k === 'loading' ? undefined : v); }));
}
exports.stripObject = stripObject;
function fetchIntrospectionSchema(url) {
    var body = JSON.stringify({ query: graphql_2.getIntrospectionQuery() });
    return fetch(url, {
        method: 'POST',
        headers: { 'Content-Type': 'application/json' },
        body: body
    })
        .then(function (res) { return res.json(); })
        .then(function (res) { return res.data; });
}
exports.fetchIntrospectionSchema = fetchIntrospectionSchema;
exports.isClass = function (fn) { return /^\s*class/.test(fn.toString()); };
//# sourceMappingURL=utils.js.map